#include <unistd.h>           /* fork/exec */
#include <stdlib.h>           /* malloc/free */
#include <time.h>
#include <ctype.h>
#include <glob.h>
#include "forms.h"
#include "fb.h"
#include "FaxBrowser.h"

char *do_alias(char *phone)
{
    FILE *fd;
    char *p;
    char line[BUFSIZ];

    if ((fd = fopen(fax_alias, "r")) != NULL) {
        while (fgets(line, sizeof(line), fd) != NULL)
            if ((p = strchr(line, '\n')) != NULL) {
		*p = '\0';
		if (line[0] == '#')
		    continue;
		if ((strncmp(line, phone, strlen(phone)) == 0) &&
		  ((p = strchr(line, ',')) != NULL)) {
		    fclose(fd);
		    p++;
		    while (isblank(*p))	     /* skip space or tab characters */
			p++;
		    return(p);
		}
            }
        fclose(fd);
    }
    return(phone);
}

char *analyze_file(char *filename)
{
    time_t ttt;
    char res;
    char tty[4];
    char number[80];
    char page[4];
    char datum[32];
    char br_entry[256];
    char *p;

    if ((p = (char *) strrchr(filename, '/')) == NULL) return(NULL);
    if((sscanf(++p, "f%c%7lx%2s-%s", &res, &ttt, tty, number)) != 4)
	return(NULL);

    /* create a valid timestamp from the mgetty timestamp */
    ttt += 0x30000000;

    if ((p = (char *) strrchr(number, '.')))
	*p = '\0';
    strcpy(page, ++p);
    p = number;

    if ((number[strlen(number)-1]) == '_') {
	number[strlen(number)-1] = '\0';
	if ((number[strlen(number)-1]) == '-')
	    number[strlen(number)-1] = '\0';
	if ((p[0] == '_'))
	    ++p;
	if ((p[0] == '-'))
	    ++p;
	strcpy(number, p);
    }
    strcpy(datum, ctime(&ttt));
    if((p = (char *) strrchr(datum, '\n'))) *p = '\0';
    strcpy(number, do_alias(number));
  
    sprintf(br_entry, "%c	%s	%s	%s", res, page, datum, number);
    p = br_entry;
    return(p); 
}

void get_filenames(void)
{
    int i, total_lines;
    char *lp;
    char xname[MAXPATHLEN];
    struct faxent *p;
    glob_t fnglob;

    total_lines = fl_get_browser_maxline(browser_fax);

    fl_freeze_form(fb);

    if (total_lines > 0) {
	for (i=1; i<=total_lines; i++) free(faxentptr[i]);
	fl_clear_browser(browser_fax);
	faxnr = 0;
    }
	
    memset(&fnglob, 0, sizeof(fnglob));
    strcpy(xname, fax_dir);
    strcat(xname, "/");
    strcat(xname, FAX_DETECT_PATTERN);
    switch(glob(xname, GLOB_ERR, NULL, &fnglob)) {
	case GLOB_NOSPACE:
	    fprintf(stderr, "glob: out of memory\n");
	    break;
	case GLOB_ABEND:
	    fl_show_alert("Alert", "Can't access fax incoming directory!",
	      "Please check your .faxbrowser.rc file!", 0);
	    break;
	case GLOB_NOMATCH:
	    fl_show_alert("Alert", "No faxes found in the fax incoming directory!",
	      "Please check your .faxbrowser.rc file!", 0);
	    break;
	default:
	    VBPRINTF("Found %d faxes\n", fnglob.gl_pathc);
	    while(fnglob.gl_pathv[0]) {
		p = (struct faxent *) malloc (sizeof(struct faxent));
		p->status = 0;
		strcpy(p->filename, *fnglob.gl_pathv++);
		if ((lp = analyze_file(p->filename))) {
		    strcpy(p->description, lp);
		    if (p->description[0] == 'n')
			p->resolution = 'n';
		    else
			p->resolution = 'f';
#ifdef HSCROLLBAR
		    /* workaround for hscrollbar bug in xforms 0.86 */
		    strcat(p->description, "                   ");
#endif
		    fl_addto_browser(browser_fax, p->description);
		    faxentptr[++faxnr] = (char *) p;
		}
	    }
	    break;
    }
    fl_unfreeze_form(fb);
}

void process_fax(char *fname, char resolution, char *mode)
{
    char fvc[512], xname[MAXPATHLEN], command[MAXPATHLEN];
    char *fp;

    if (!strcmp(mode, "view")) {
	if (resolution == 'f')
	    strcpy(command, fax_viewcommandf);
	else
	    strcpy(command, fax_viewcommandn);
    } else {
	if (resolution == 'f')
	    strcpy(command, fax_printcommandf);
	else
	    strcpy(command, fax_printcommandn);
    }

    strcpy(fvc, command);
    if ((fp = (char *) strrchr(fvc, '#'))) {
	*fp = '\0';
	sprintf(xname, "%s%s%s", fvc, fname, ++fp);
    } else {
	sprintf(xname, "%s %s", command, fname);
    }
    VBPRINTF("%sing: %s\n", mode, xname);

    if (!fork()) {
	execl("/bin/sh", "sh", "-c", xname, (char *) 0);
    }
}

void get_selection(void)
{
    struct faxent *p;
    int i, total_lines;

    total_lines = fl_get_browser_maxline(browser_fax);

    for (i = 1; i <= total_lines; i++) {
	p = (struct faxent *) faxentptr[i];
	if (fl_isselected_browser_line(browser_fax, i)) {
	    p->status = 1;
	} else {
	    p->status = 0;
	}
    }
}

